"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const hyperscript_1 = __importDefault(require("hyperscript"));
const indent_utils_1 = require("indent-utils");
const eqdict_1 = __importDefault(require("@patarapolw/eqdict"));
const tokenize_1 = require("./tokenize");
class HyperPug {
    constructor(filters = {}) {
        this.filters = filters;
    }
    parse(s) {
        return hyperscript_1.default('div', this.precompile(s)).innerHTML;
    }
    precompile(s) {
        let key = '';
        let childrenRows = [];
        const nodes = [];
        let isInFilter = false;
        for (const r of indent_utils_1.stripIndent(s).split('\n')) {
            if (r[0] && r[0] !== ' ') {
                isInFilter = false;
            }
            if (/\S/.test(r[0]) && !isInFilter) {
                if (r[0] === ':') {
                    isInFilter = true;
                }
                if (key) {
                    nodes.push(this.generate(key, childrenRows));
                    childrenRows = [];
                }
                key = r;
                continue;
            }
            childrenRows.push(r);
        }
        if (key) {
            nodes.push(this.generate(key, childrenRows));
        }
        return nodes;
    }
    generate(key, childrenRows) {
        const c = childrenRows.join('\n');
        const children = c ? this.precompile(c) : undefined;
        if (key[0] === ':') {
            return this.buildH(key, null, indent_utils_1.stripIndent(c));
        }
        let d = null;
        if (key[0] === ':') {
            return this.buildH(key, null, indent_utils_1.stripIndent(c));
        }
        const { key: k1, dict, suffix, content } = tokenize_1.tokenize(key);
        if (dict) {
            d = eqdict_1.default(dict);
        }
        if (suffix === '.') {
            return this.buildH(k1, d, indent_utils_1.stripIndent(c));
        }
        else if (suffix === ':') {
            return this.buildH(k1, d, this.precompile(content));
        }
        return this.buildH(k1, d, content || children);
    }
    buildH(key, attrs, children) {
        if (key[0] === ':') {
            const filterName = key.substr(1);
            const fn = this.filters[filterName];
            if (!fn) {
                throw new Error(`Filter not installed: ${filterName}`);
            }
            if (typeof children !== 'string') {
                throw new Error(`Nothing to feed to filter: ${filterName}`);
            }
            return hyperscript_1.default('div', { innerHTML: fn(children) });
        }
        attrs = attrs || {};
        for (const [k, v] of Object.entries(attrs)) {
            if (!v) {
                attrs[k] = '';
            }
        }
        try {
            return hyperscript_1.default(key, { attrs: attrs || {} }, children);
        }
        catch (e) {
            return hyperscript_1.default('div', { attrs: attrs || {} }, children);
        }
    }
}
exports.default = HyperPug;
//# sourceMappingURL=index.js.map